;----------------------------------------------------------------------------
;     MODULE NAME:   GUESSFAT.IH
;
;         $Author:   USER "Dennis"  $
;       $Revision:   1.0  $
;           $Date:   04 Feb 2002 18:14:26  $
;        $Logfile:   C:/DBAREIS/Projects.PVCS/PpwAddOn/GUESSFAT/GuessFat.h.pvcs  $
;
;     DESCRIPTION
;     ~~~~~~~~~~~
;     Given a long filename (with or without slashes) will
;     return a FAT filename (~1's etc).
;
;     Note that Thanks to Bill it is not actually possible to
;     determine the short name for a non-existant file but here we
;     make a reasonable guess. It should be noted that it is a guess
;     and it is impossible to do better. As with a lot of things
;     Windows, results will depend of your luck.
;
;     GuessFatName - Not for use within  #evaluate etc
;     ~~~~~~~~~~~~
;     SHORT = Name of a macro to receive the short name
;             (rexx variable with the same name is also set)
;     LONG  = The actual long path.
;
;     _GuessFatName - For use within an "#evaluate" etc.
;     ~~~~~~~~~~~~~
;
;     SHORT = Name of a rexx variable to receive the short name.
;     LONG  = Name of a rexx variable (or literal) containing the
;             actual long path.
;
;----------------------------------------------------------------------------


;--- Only include once! -----------------------------------------------------
#ifdef VERSION_GUESSFAT_H
       #eof 1                      ;;No point reading rest of file!
#endif
#define VERSION_GUESSFAT_H    02.035

;--- Create our own namespace -----------------------------------------------
#Nextid

;--- Macro for use outside of REXX ------------------------------------------
#define? GUESSFAT_REDFINE_OK N             ;;global default
#define  GuessFatName \
         #evaluate ^^ ^<$_GuessFatName SHORT=^{$SHORT}^ LONG=^"{$LONG}"^ >; call MacroSet "{$SHORT}", {$SHORT}, "{$RedefineOK='<$GUESSFAT_REDFINE_OK>'}";^

;--- Macro for use within rexx code -----------------------------------------
#DefineRexx '_GuessFatName'
   /*--- Work through each directory component ------------------------------*/
   @@Long   = {$Long};
   {$Short} = ''
   do  while @@Long \== ''
       /*--- Get Next bit ---------------------------------------------------*/
       parse var @@Long @@Bit '\' @@Long;

       /*--- Remove spaces --------------------------------------------------*/
       @@Bit = ReplaceString(@@Bit, " ", "");

       /*--- Look for the last dot ------------------------------------------*/
       @@DotPos = lastpos('.', @@Bit);
       if  @@DotPos = 0 then
           @@Extn = "";
       else
       do
           /*--- Extn includes the dot --------------------------------------*/
           @@Extn = substr(@@Bit, @@DotPos);
           @@Bit = left(@@Bit, @@DotPos-1)

           /*--- Only want first 3 characters in the extension --------------*/
           if  length(@@Extn) > 4 then
               @@Extn = left(@@Extn, 4);

           /*--- No dots in first bit! --------------------------------------*/
           @@Bit = ReplaceString(@@Bit, ".", "")
       end;

       /*--- Now Look at the @@Bit and see if we need "~1" ------------------*/
       if  length(@@Bit) > 8 then
           @@Bit = left(@@Bit, 6) || "~1"

       /*--- Add to short path ----------------------------------------------*/
       if  {$Short} \== "" then
           {$Short} = {$Short} || '\';
       {$Short} = translate( {$Short} || @@Bit || @@Extn );
   end;
#DefineRexx
