;----------------------------------------------------------------------------
;     MODULE NAME:   NEEDGUID.H
;
;         $Author:   USER "Dennis"  $
;       $Revision:   1.3  $
;           $Date:   12 Apr 2001 17:40:02  $
;        $Logfile:   C:/DBAREIS/Projects.PVCS/PpwAddOn/NeedGuid/NeedGuid.h.pvcs  $
;
;     DESCRIPTION:   This header simplifies the creation of GUIDs (and
;                    can maintain persistance if required).
;
;                    See the tail end of this file for some documentation
;                    and examples.
;----------------------------------------------------------------------------

;--- Define version number of this header -----------------------------------
#ifdef  NEEDGUID_VERSION
        ;--- Can't be silent, user may be trying to set options -------------
        #error ^You have already included this header file!^
#endif
#define NEEDGUID_VERSION    01.102

;--- Defaults ---------------------------------------------------------------
#define? NEEDGUID_DEFAULT_FORMAT      REGISTRY ;;Default conversion format
#define? NEEDGUID_DEFAULT_LOADNOW     Y        ;;Load now for generate "#include" into output file?
#define? NEEDGUID_GUID_FILE_PREFIX
#define? NEEDGUID_GUID_FILE_SUFFIX    .guid
#define? NEEDGUID_HANDLE_NEW_FORMATS           ;;Rexx code to handle user defined formats

;--- Constants --------------------------------------------------------------
#define NEEDGUID_GENERIC_GUID_LNG    36
#define NEEDGUID_GENERIC_FORMAT      ''


;----------------------------------------------------------------------------
;--- How to get a "persistant" GUID (once created, remains constant) --------
;----------------------------------------------------------------------------
#define GuidGet                                                 \
        #evaluate ^^ ^<$Rexx4GuidGet GUIDVAR="TmpGuid" {$?}>^  -\
        <??TmpGuid>
#ifndef Rexx4GuidGet
   #NextId
   #DefineRexx 'Rexx4GuidGet' NOPACK
       ;--- Use what file to remember value ---------------------------------
       @@File = '{$FILE=^^}';
       if  @@File = '' then
       do
           ;--- User does not care what we call it --------------------------
           @@File = '<$NEEDGUID_GUID_FILE_PREFIX>{$Alias}<$NEEDGUID_GUID_FILE_SUFFIX>';
           @@File = ReplaceString(@@File, ' ', '_');
           @@File = '{$PATH=^.^}\' || @@File;
       end;

       ;--- Do we already have a value stored? ------------------------------
       if  stream(@@File, 'c', 'query exists') <> '' then
       do
           ;--- Get stored value --------------------------------------------
           call AddInputFileToDependancyList @@File;
           @@Generic = linein(@@File, 1);
           @@Rc      = stream(@@File, 'c', 'close');

           ;--- Make sure the retrieved value looks OK ----------------------
           if  length(@@Generic) <> <$NEEDGUID_GENERIC_GUID_LNG> then
           do
               error('The GUID in "' || @@File || '" does not appear to be in a generic format', 'Line1: ' || @@Generic);
           end;
       end;
       else
       do
           ;--- "Fixed" GUID does not exist, generate a new GUID-------------
           <$Rexx4GuidCreate GENVAR="@@Generic">

           ;--- Work out a comment ------------------------------------------
           @@CmtLine1 = 'MADE AT : ' || date() || ', ' || GetAmPmTime();
           @@CmtLine2 = 'LOCATION: ' || GetInputFileNameAndLine();
           @@CmtLine3 = 'USER CMT: {$COMMENT="None" $$SQx2}';

           ;--- Store the result for next time ------------------------------
           call lineout @@File, @@Generic;         ;;The guid itself
           call lineout @@File, ''
           call lineout @@File, @@CmtLine1;        ;;Creation Time
           call lineout @@File, @@CmtLine2;        ;;Location in user source
           call lineout @@File, @@CmtLine3;        ;;Any user comment (reason for GUID etc)
           call DieIfIoErrorOccurred @@File;
           @@Rc = stream(@@File, 'c', 'close');

           ;--- If this file changes we will need to rebuild! So mark as INPUT dependancy! ---
           call AddInputFileToDependancyList @@File;
       end;

       ;--- Convert Generic format into desired format ------------------
       <$Rexx4GuidConvert GENVAR="@@Generic" GUIDVAR="@@GuidFormatted" FORMAT="{$FORMAT='<$NEEDGUID_DEFAULT_FORMAT>'}">

       ;--- Return the value OR define it -----------------------------------
       @@Define = '{$DEFINE=^^}';
       if  @@Define = '' then
           {$GUIDVAR="GuidFixed"} = @@GuidFormatted;
       else
       do
           ;--- Just create a define ----------------------------------------
           {$GUIDVAR} = '';
           call MacroSet @@Define, @@GuidFormatted;
       end;
   #DefineRexx
#endif

;----------------------------------------------------------------------------
;--- How to create a NEW GUID -----------------------------------------------
;----------------------------------------------------------------------------
#define GuidCreate                                                    \
        #evaluate ^^ ^<$Rexx4GuidCreate  GENVAR="TmpGuid" {$?}>; <$Rexx4GuidConvert GENVAR="TmpGuid" FORMAT="{$FORMAT='<$NEEDGUID_DEFAULT_FORMAT>'}" GUIDVAR="TmpGuid">^ -\
        <??TmpGuid>
#ifndef Rexx4GuidCreate
   #NextId
   #DefineRexx 'Rexx4GuidCreate' NOPACK
       ;--- Need a temporary file -------------------------------------------
       @@TmpFileGuid   = RexGetTmpFileName('GUID????.TMP');
       @@TmpFileOutput = RexGetTmpFileName('GUIDO???.TMP');

       ;--- Generate the GUID -----------------------------------------------
       @@Cmd = 'uuidgen.exe "-o' || @@TmpFileGuid || '"';
       @@Rc = AddressCmd(@@Cmd || ' > "' || @@TmpFileOutput || '" 2>&1');
       if  @@Rc <> 0 then
       do
           ;--- Program failed ----------------------------------------------
           call Say '';
           call Say @@Cmd;
           call AddressCmd('type "' || @@TmpFileOutput || '"');
           call _SysFileDelete @@TmpFileGuid;
           call _SysFileDelete @@TmpFileOutput;
           error('Could not generate GUID using "uuidgen.exe"', 'RC = ' || @@Rc);
       end;

       ;--- Read the Guid ---------------------------------------------------
       {$GENVAR='GuidGeneric'} = linein(@@TmpFileGuid, 1)
       @@Rc        = stream(@@TmpFileGuid, 'c', 'close');
       call _SysFileDelete @@TmpFileGuid;
       if  length({$GENVAR}) <> <$NEEDGUID_GENERIC_GUID_LNG> then
       do
           ;--- Failed (even though RC from program was OK) -----------------
           call Say '';
           call Say @@Cmd;
           call AddressCmd('type "' || @@TmpFileOutput || '"');
           call _SysFileDelete @@TmpFileOutput;
           error('Could not generate GUID using "uuidgen.exe" (Return code was zero!)', 'Got: ' || {$GENVAR});
       end;

       ;--- GUID OK so won't need to see output! ----------------------------
       call _SysFileDelete @@TmpFileOutput;
   #DefineRexx
#endif


;----------------------------------------------------------------------------
;--- Convert "generic" guid format into desired -----------------------------
;----------------------------------------------------------------------------
#ifndef Rexx4GuidConvert
   #NextId
   #DefineRexx 'Rexx4GuidConvert' NOPACK
       ;--- Use passed GUID or get from global variable ---------------------
       @@Generic = {$GENVAR='GuidGeneric'};
       if  length(@@Generic) <> <$NEEDGUID_GENERIC_GUID_LNG> then
       do
           error('The passed GUID does not appear to be in a generic format', 'PASSED: ' || @@Generic);
       end;

       ;--- Perform the conversion (format is case sensitive) ---------------
       @@Fmt = '{$FORMAT='<$NEEDGUID_DEFAULT_FORMAT>'}'
       select
           /*+++++++++++++++++++++++++++++++*/
           when @@Fmt = <$NEEDGUID_GENERIC_FORMAT> then
           /*+++++++++++++++++++++++++++++++*/
           do
               ;--- Leave as it is (no conversion) --------------------------
               @@Formatted = @@Generic;
           end;
           /*+++++++++++++++++++++++++++++++*/
           when @@Fmt = 'REGISTRY' then
           /*+++++++++++++++++++++++++++++++*/
           do
               ;--- Upper case and in curley brackets -----------------------
               @@Formatted = '{' || translate(@@Generic) || '}';
           end;
           /*+++++++++++++++++++++++++++++++*/
           when @@Fmt = 'registry' then
           /*+++++++++++++++++++++++++++++++*/
           do
               ;--- Upper case and in curley brackets -----------------------
               @@Formatted = '{' || ToLowerCase(@@Generic) || '}';
           end;

           ;--- Let user add their own format -------------------------------
           <$NEEDGUID_HANDLE_NEW_FORMATS>

           /*+++++++++++++++++++++++++++++++*/
           otherwise
           /*+++++++++++++++++++++++++++++++*/
           do
               error("Don't" || ' know how to convert to the "' || @@Fmt || '" GUID format');
           end;
       end;

       ;--- "return" the result ---------------------------------------------
       {$GUIDVAR="GuidFormatted"} = @@Formatted;
   #DefineRexx
#endif


;--- We will keep doco at end of this file so it does not get processed -----
#EOF


;----------------------------------------------------------------------------
;---------------------------[ SOME DOCUMENTATION ]---------------------------
;----------------------------------------------------------------------------
;
;  The following does not even begin to attempt to document
;  everything you can do, but between this and the sample code
;  you should not have any problems. More advanced users can
;  have a closer look at the macros etc to work out other
;  options/features.
;
;
;  GuidCreate
;  ~~~~~~~~~~
;  This macro generates generate a new GUID every time it is used.
;  The GUID is created in the format specified by the "FORMAT"
;  parameter.
;
;
;  GuidGet
;  ~~~~~~~
;  This macro will return the SAME GUID every time it is used.
;  The "ALIAS" parameter should uniquely describe the guid.
;
;  The GUID is stored in a generic format and formatted as
;  per the "FORMAT" parameter.
;
;  By default the files used to store GUID information are
;  created in the current directory and should probably be
;  considered as source files (backed up and under revision
;  control etc).
;  The reason you are using this macro is to ENSURE
;  that you keep one or more GUIDs fixed over an
;  indefinite period of time.
;  If you need for "force" the use of a new value
;  simply delete the ".guid" file for that GUID.
;
;
;  SOME EXAMPLE CODE
;  ~~~~~~~~~~~~~~~~~
;  #include "NEEDGUID.IH"
;
;  A Random GUID (different each time)
;  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;  FORMAT - GENERIC :  <$GuidCreate FORMAT="">
;  FORMAT - REGISTRY: <$GuidCreate>
;  FORMAT - REGISTRY: <$GuidCreate FORMAT="REGISTRY">
;  FORMAT - registry: <$GuidCreate FORMAT="registry">
;
;
;  <?Newline>
;  A Random GUID (same GUID every time!)
;  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;  FIXED - REGISTRY <$GuidGet ALIAS="ProductId">
;----------------------------------------------------------------------------
