;----------------------------------------------------------------------------
;--- This header file is (C)opyright Dennis Bareis 2001. All rights reserved.
;----------------------------------------------------------------------------
;
;     MODULE NAME:   COUNTER.H
;
;         $Author:   USER "Dennis"  $
;       $Revision:   1.1  $
;           $Date:   01 Mar 2001 17:58:00  $
;        $Logfile:   C:/DBAREIS/Projects.PVCS/PpwAddOn/Counter/counter.h.pvcs  $
;
;     DESCRIPTION:   See End of this file.
;
;----------------------------------------------------------------------------
;---              USAGE Details (see end of this file)
;----------------------------------------------------------------------------


;--- Define version number of this header -----------------------------------
#define  VERSION_COUNTER_H   01.060

;--- This is the macro the user will use ------------------------------------
#define  Counter                                \
         #evaluate ^^ ^<$Rexx4Counter {$?}>^   -\
         <??CounterCode>

;--- Set up user defaults ---------------------------------------------------
#define? COUNTER_DEFAULT_START                 1
#define? COUNTER_VALUE   \
         {$ValueAfterVar} = {$ValueBeforeVar}  ;;A decimal number (unchanged)

;--- Used internally to increment counter and generate digits (default as text) ---
#NextId
#DefineRexx "Rexx4Counter"
            ;--- Have a value we wish to set the counter to? ----------------
            @@CountersVar = 'CNTR__' || c2x('{$#1="DEFAULT"}');
            @@NextVal     = '{$VALUE=""}';
            if @@NextVal <> '' then
            do
               ;--- Have a value we wish to use -----------------------------
               @@Value = @@NextVal;
            end;
            else
            do
               ;--- Want to increase stored value ---------------------------
               if  symbol(@@CountersVar) <> 'VAR' then
               do
                   ;--- No stored value and no starting value supplied (use default) --
                   @@Value = <$COUNTER_DEFAULT_START>;
               end;
               else
               do
                   ;--- Get current value and increment ---------------------
                   @@Value = value(@@CountersVar) + 1;
               end;
            end;

            ;--- Update the counter -----------------------------------------
            call value @@CountersVar, @@Value;

            ;--- What to do with it (COUNTER_DIGIT takes priority) -----------------------
            #ifndef COUNTER_DIGIT
               ;--- Processing is done on whole value (not individual digits) ---
               <$COUNTER_VALUE NAME=^{$#1}^ ValueBeforeVar="@@Value" ValueAfterVar="CounterCode">
            #elseif
               ;--- Generate a digit at a time ("COUNTER_DIGIT" was defined) ---
               CounterCode = '';
               do until  @@Value = ''
                   ;--- Get next digit --------------------------------------
                   @@Digit = left(@@Value, 1);
                   @@Value = substr(@@Value, 2);

                   ;--- Convert digit and add to output ---------------------
                   <$COUNTER_DIGIT NAME=^{$#1}^ DigitVar="@@Digit" DigitCodeVar="@@DigitCode">
                   CounterCode = CounterCode || @@DigitCode;
               end;
            #endif
#DefineRexx


#eof



                           ********************
                           *** INSTRUCTIONS ***
                           ********************



Introduction
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
This header is can be used to have one or more counters. Each time the
counter is used it is incremented by one. By default the counter simply
returns the value of the incremented counter as text (for example "12")
however if you have graphics digits it can return the "IMG" tag for
for "1" followed by that for "2".

If you need formatting/alignment (for example to simulate a numbered
list) then you would probably use a table and  whatever alignment you
wanted on the counter cell.

If you need more than one counter at a time you can name your counter.
The NAME is case sensitive with the default being "DEFAULT".

You can set the value for a counter with the "VALUE" parameter.

If a counter does not exist and no value is supplied then it is created
and by default the counter starts at one.

Note that the name of the counter is also passed to the configurable
rexx code that creates the output so your generation could be selective
based on the counter name.


Example
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
<br><$Counter> - 1
<br><$Counter> - 2
<br><$Counter> - 3
<br><$Counter value="9"> - 9
<br><$Counter> - 10
<br><$Counter value="666"> - 666
<br><$Counter "AA"> - 1 (AA)
<br><$Counter> - 667
<br><$Counter> - 668
<br><$Counter "AA"> - 2 (AA)
<br><$Counter> - 669
