;----------------------------------------------------------------------------
;     MODULE NAME:   DOTDOT.H
;
;         $Author:   USER "Dennis"  $
;       $Revision:   1.1  $
;           $Date:   26 Oct 2001 21:04:26  $
;        $Logfile:   C:/DBAREIS/Projects.PVCS/MultiOs/PPWIZARD/dotdot.h.pvcs  $
;
;     DESCRIPTION:   If you process many subdirectories at once but have
;                    a fixed directory you wish to point to then this header
;                    file will help.
;
;                    For example you might have one or more directory trees
;                    containing html code which all load images from a
;                    common graphics directory. Sure you could code an
;                    absolute address for each image but you wish the
;                    flexability of keeping relative filenames/URLS.
;
;                    Note that this is a general purpose header file useful
;                    for accessing local files as well as for use in html
;                    pages.
;
;                    The macro will calculate the number of directories you
;                    are from a root directory you specify, if most or all
;                    of your files will be from a common root you can define
;                    the "DOTDOT_ROOT" variable so that you don't have to use
;                    the "ROOT" parameter when using the macro.
;
;                    If you are creating local file system filenames (and
;                    not for unix) then you might wish to define
;                    "DOTDOT_SLASH" as follows:
;
;                            #define DOTDOT_SLASH    \
;
;         EXAMPLE (SIMPLE):
;         ~~~~~~~~~~~~~~~~~
;
;         #define  DOTDOT_ROOT     E:\DB\
;         #include "dotdot.h"
;
;         To get to "E:\DB\"
;         ~~~~~~~~~~~~~~~~~~
;         #evaluate ^^ ^<$DotDotSlash FILE="E:\DB\DIR1\file">^
;         E:\DB\DIR1\file = "<??DotDot>"
;
;         #evaluate ^^ ^<$DotDotSlash ROOT="E:\DB\" FILE="E:\DB\file">^
;         E:\DB\file      = "<??DotDot>"
;
;         #evaluate ^^ ^<$DotDotSlash FILE="E:\DB\DIR1\DIR2\file" SLASH="\">^
;         E:\DB\DIR1\DIR2\file = "<??DotDot>"
;
;         To get to "E:\DB\graphics"
;         ~~~~~~~~~~~~~~~~~~~~~~~~~
;         #evaluate ^^ ^<$DotDotSlash FILE="E:\DB\DIR1\reddot.gif">^
;         E:\DB\DIR1\file = "<??DotDot>graphics/reddot.gif"
;
;         EXAMPLE (MORE REALISTIC):
;         ~~~~~~~~~~~~~~~~~~~~~~~~~
;         This example is simple and more realistic than the above (although
;         not very efficient as the root path is calculated as many times
;         as required rather than just the once for each html page).
;         This example assumes that html pages can be in many levels off
;         a root and that they need to access common resources such as
;         the "/images" directory.
;
;         ;--- Define "ToRoot" macro (path to root) --------------------
;         #define+ DOTDOT_ROOT   E:\DB\HOMEPAGE\
;         #include "dotdot.h"
;         #define  ToRoot                                             \
;                  #evaluate ^^ ^<$DotDotSlash FILE="<?InputFile>">^ -\
;                  <??DotDot>
;
;         ;--- Use a graphic -------------------------------------------
;         <IMG SRC="<$ToRoot>images/paper.gif">
;
;----------------------------------------------------------------------------


;--- Only include once ------------------------------------------------------
#ifndef VERSION_DOTDOT_H
        #define   VERSION_DOTDOT_H    01.293


;--- Allow user to specify default root -------------------------------------
#ifndef DOTDOT_ROOT
        ;--- User must specify "ROOT" param on all uses of "DotDot" macro! --
        #define DOTDOT_ROOT     root parameter not specified!!
#endif


;--- Allow user to specify default slash ------------------------------------
#ifndef DOTDOT_SLASH
        ;--- User must specify "ROOT" param on all uses of "DotDot" macro! --
        #define DOTDOT_SLASH    /          ;;Good for unix and web addresses
#endif


;-- Assume passed "FILE" parameter is a FILE, if a dir you will need to
;-- add another "..\" (as last part ignored).
#DefineRexx "DotDotSlash"
            ;--- Get passed parameters (convert back slashes to forward) ----
            RxYourFile = ReplaceString('{$File $$SQx2}',                  '\', '/');
            RxYourRoot = ReplaceString('{$Root="<$DOTDOT_ROOT>" $$SQx2}', '\', '/');

            ;--- If not linux make upper case (so case insensitive) ---------
            #if  ['<?OpSys>' <> 'UNIX']
                 RxYourFile = translate(RxYourFile);
                 RxYourRoot = translate(RxYourRoot);
            #endif

            ;--- Make sure that this is off the "root" ----------------------
            RxYourRootL = length(RxYourRoot);
            if left(RxYourFile, RxYourRootL) <> RxYourRoot then
               error('The file "' || RxYourFile || '" is not off the root directory "' || RxYourRoot || '"');

            ;--- Get relative bit (want '..\' for each dir left) ------------
            RxYourFile = substr(RxYourFile, RxYourRootL+1)
            DotDot   = '';
            FPos = 1;
            SPos = pos('/', RxYourFile, FPOS);
            do while SPos <> 0
               ;--- Found another slash -------------------------------------
               DotDot = DotDot || '..{$Slash="<$DOTDOT_SLASH>"}';

               ;--- Look again ----------------------------------------------
               FPos = SPos + 1;
               SPos = pos('/', RxYourFile, FPOS);
            end;
#DefineRexx


;--- End of Header file -----------------------------------------------------
#endif

