;----------------------------------------------------------------------------
;     MODULE NAME:   EXECSYS.H
;
;         $Author:   USER "Dennis"  $
;       $Revision:   1.2  $
;           $Date:   24 Mar 2001 14:43:44  $
;        $Logfile:   C:/DBAREIS/Projects.PVCS/PpwAddOn/ExecSys/execsys.h.pvcs  $
;
;     DESCRIPTION:   This is a header file for handling inclusion of
;                    perl/VbScript/C code output into HTML pages.
;
;                    Macro "Exec2File"
;                    ~~~~~~~~~~~~~~~~~
;                    This macro takes the following parameters:
;
;                         FILE
;                         ~~~~
;                         Optional. If not supplied will generate a random
;                         file in the "TMP" directory. The filename is always
;                         returned in the "Exec2File" rexx variable.
;                         You would normally delete the file when you have
;                         finished with it.
;
;                         CMD
;                         ~~~
;                         Manditory. This holds the command you wish to
;                         execute WITHOUT any output redirection commands.
;                         It will be executed and the output placed into a
;                         file. The return code will be available in the
;                         "Exec2FileRc" rexx variable. Note that by default
;                         ppwizard will terminate with an error if the
;                         return code is not 0.
;
;                         ERROR
;                         ~~~~~
;                         Optional. This is the test on the return code
;                         (in rexx variable "RC") that is used to determine
;                         when ppwizard should terminate with an error.
;                         The default is "Rc <> 0".
;                         If the command fails ppwizard normally deletes the
;                         file containing the output, it does not do this if
;                         debug is on.
;
;
;                    Macro "LoadFile"
;                    ~~~~~~~~~~~~~~~
;                    This macro loads a file into rexx variable(s).
;                    It will either load the complete file info a specified
;                    variable or into an array.
;
;                    This macro takes the following parameters:
;
;                         FILE
;                         ~~~~
;                         Optional. If not supplied will use the name of the
;                         last file created with the "Exec2File" macro.
;
;                         VARIABLE
;                         ~~~~~~~~
;                         Optional. If not supplied then it is assumed you
;                         wish the file loaded line by line into a rexx array.
;                         If specified specifies the name of the rexx
;                         variable you wish to hold the contents in.
;
;                         ARRAY
;                         ~~~~~
;                         Optional. Has no effect if "VARIABLE" was used.
;                         If not supplied defaults to "LOADFILE".
;                         Note that the value should not end with a dot!
;
;
;                    Macro "Exec2FileDelete"
;                    ~~~~~~~~~~~~~~~~~~~~~~~
;                    This macro takes no parameters and simply deletes the
;                    last file created with the "Exec2File" macro. It ignores
;                    any deletion errors.
;----------------------------------------------------------------------------


;----------------------------------------------------------------------------
;--- Define the version number of this header file --------------------------
;----------------------------------------------------------------------------
#ifdef    VERSION_EXECSYS_H
          ;--- Only need to include once! -----------------------------------
          #eof 1
#endif
#define   VERSION_EXECSYS_H    01.083
#require  01.083                        ;;We are marking temporary files


;----------------------------------------------------------------------------
;--- Execute the command (redirecting all output) ---------------------------
;----------------------------------------------------------------------------
#NextId
#define Exec2File                               \
        #evaluate ^^ ^<$Rexx4Exec2File {$?}>^
#DefineRexx 'Rexx4Exec2File'
   ;--- Get parameters ------------------------------------------------------
   @@Cmd     = '{$Cmd $$SQx2}';
   Exec2File = '{$File=^^}';               ;;Global variable "returned" by macro
   if  Exec2File = '' then
   do
       ;--- User did not supply filename so create one ----------------------
       Exec2File = RexGetTmpFileName('E2F_????.TMP');
   end;

   /*--- Mark the output file as a temporary one! ---------------------------*/
   call AddTempFileToDependancyList Exec2File;

   ;--- Work out how to redirect output -------------------------------------
   @@Cmd = @@Cmd || RedirectStdOutAndErr2(Exec2File);

   ;--- Execute the command -------------------------------------------------
   Exec2FileRc =  AddressCmd(@@Cmd);       ;;"Exec2FileRc" is a global variable "returned" by macro
   Rc          =  Exec2FileRc;

   ;--- Check Return code (default test is RC = 0 for OK) -------------------
   if  {$ERROR=^RC <> 0^} then
   do
       ;--- Delete file unless user in debug mode ---------------------------
       if  IsDebugOn() = 'N' then
       do
           ;--- Debug is not on so get rid of temporary file ----------------
           call _SysFileDelete Exec2File
       end;

       ;--- Exit with error message -----------------------------------------
       call Error 'Failed executing "{$Cmd $$SQx2}"!, RC: ' || Exec2FileRc || ', Error When: {$ERROR}'
   end;
#DefineRexx
#RexxVar Exec2File = ''                ;;Initialize variable


;----------------------------------------------------------------------------
;--- Delete the last file used (unless debug is on, errors ignored) ---------
;----------------------------------------------------------------------------
#define Exec2FileDelete                               \
        #evaluate ^^ ^<$Rexx4Exec2FileDelete>^
#DefineRexx 'Rexx4Exec2FileDelete'
           ;--- Make sure variable defined, if so delete file ---------------
           if  symbol('Exec2File') = 'VAR' then
           do
               ;--- Delete the named file -----------------------------------
               call _SysFileDelete Exec2File;
           end;
#DefineRexx

;----------------------------------------------------------------------------
;--- Load the file into memory ----------------------------------------------
;----------------------------------------------------------------------------
#define LoadFile                                       \
        #evaluate ^^ ^<$Rexx4LoadFile {$?}>^
#define? EXECSYS_LOADFILE_DEFAULT_ARRAY_NAME  LOADFILE
#DefineRexx 'Rexx4LoadFile'
   ;--- Get passed filename an make sure file is closed ---------------------
   @@File = '{$FILE=^<??Exec2File>^}';
   @@Rc = stream(@@File, 'c', 'close');

   ;--- Just load whole file into variable or individual lines into a "stem"? ---
   if  '{$VARIABLE=^NOTPASSED^}' <> 'NOTPASSED' then
   do
       ;--- User wished to load file into a variable ------------------------
       @@FileLng   = stream(@@File, 'c', 'query size');
       {$VARIABLE} = charin(@@File, 1, @@FileLng);      ;;CR, LF and other characters left exactly as they are in the file
   end;
   else
   do
       ;--- Load individual lines into the specified array ------------------
       @@Lines = 0;
       do  while lines(@@File) <> 0
           ;--- Read line ---------------------------------------------------
           @@Lines = @@Lines + 1;
           @@Line = linein(@@File);

           ;--- Add to array ------------------------------------------------
           {$ARRAY=^<$EXECSYS_LOADFILE_DEFAULT_ARRAY_NAME>^}.@@Lines = @@Line;
       end;

       ;--- Set up count ----------------------------------------------------
       {$ARRAY}.0 = @@Lines;
   end;

   ;--- Close the file ------------------------------------------------------
   call DieIfIoErrorOccurred @@File;
   @@Rc = stream(@@File, 'c', 'close');
#DefineRexx
