;----------------------------------------------------------------------------
;     MODULE NAME:   FILEINFO.H
;
;         $Author:   USER "Dennis"  $
;       $Revision:   1.0  $
;           $Date:   30 Mar 2001 18:05:28  $
;        $Logfile:   C:/DBAREIS/Projects.PVCS/MultiOs/PPWIZARD/fileinfo.h.pvcs  $
;
;     DESCRIPTION:   This header allows you to easily verify the existance
;                    of a local file, and assuming you have a foolproof way
;                    of transferring changes to your ISP's server (that is
;                    you have automated the process using a product such as
;                    "sitecopy") then this will have validated the existance
;                    on the server as well (at the very least you have not
;                    miskeyed the name of the download etc).
;
;                    The header makes NO assumptions about the position on
;                    the server versus its position on the local filesystem.
;                    If the relative position on the local filesystem (to the
;                    current directory) was the same as the relative position
;                    on the server from the generated html then a path need
;                    not be specified.  If you do need to specify a path it
;                    can either be with the file (and can therefore handle
;                    more than one local path) or you can make use of the
;                    default path definition ("FILEINFO_DEFAULT_PATH").
;                    If the path definition is supplied it must end with a
;                    slash.
;
;                    If all your file information is to be generated in a
;                    different font (or bold, italic etc) then you can make
;                    use of the "FONT" related definitions, by default they
;                    are only suitable for html generation and make the font
;                    slightly smaller.  You can also create empty values if
;                    you don't wish the font to change.  You could create
;                    your own "front end" such as this one (for html):
;
;                         #define  BoldSizeOfFile <B><$SizeOfFile {$?}></B>
;
;                    Note that there are 2 levels of default, the #define
;                    values set the default for all the macros while the
;                    "SFONT" and "EFONT" parameters can be used to specify
;                    the complete start and end font tags on an individual
;                    use.
;----------------------------------------------------------------------------


;--- Start of Header (include only once) ------------------------------------
#ifndef VERSION_FILEINFO_H
        #define   VERSION_FILEINFO_H    01.053
        #require  2K.268


;--- Defaults ---------------------------------------------------------------
#define? FILEINFO_DEFAULT_PATH             ;;No path by default (note filename can contain all or partial path)
#define? FILEINFO_FONT       <FONT SIZE="-1">   ;;Can be empty for no font change!
#define? FILEINFO_/FONT      </FONT>            ;;Can be empty for no font change!
#define? FILEINFO_AM         am                 ;;Change to whatever you wish
#define? FILEINFO_PM         pm                 ;;Change to whatever you wish
#define? FILEINFO_DATE_SPACE <?Space>

;--- Define variables so they can be safely referred to by user of header ---
#ifndef    FILEINFO_RXVAR_FULL_LOCAL_FILE
           #define FILEINFO_RXVAR_FULL_LOCAL_FILE  RxLfName
#endif
#ifndef    FILEINFO_RXVAR_FILE_DATE
           #define FILEINFO_RXVAR_FILE_DATE        RxLfDate
#endif
#ifndef    FILEINFO_RXVAR_FILE_TIME
           #define FILEINFO_RXVAR_FILE_TIME        RxLfTime
#endif
#ifndef    FILEINFO_RXVAR_FILE_SIZE
           #define FILEINFO_RXVAR_FILE_SIZE        RxLfSize
#endif
#ifndef    FILEINFO_RXVAR_FILE_SIZE_CMA
           #define FILEINFO_RXVAR_FILE_SIZE_CMA    RxLfSizeCma
#endif


;--- Code to validate and format file information ---------------------------
#DefineRexx RexxVerifyLocalInputFile '$TRACE'
            ;--- Look for the file ------------------------------------------
            Look4This = '{$File}';
            LookWhere = '{$PATH=`<$FILEINFO_DEFAULT_PATH>`}'
            #if '<?OpSys>' <> 'UNIX'
                ;--- User supplied bit may be in URL form (convert fwd slashes)
                Look4This = ReplaceString(Look4This, "/", "\");
                LookWhere = ReplaceString(LookWhere, "/", "\");
            #endif
            if LookWhere = '' then
            do
               <$FILEINFO_RXVAR_FULL_LOCAL_FILE> = '';
            end;
            else
            do
               <$FILEINFO_RXVAR_FULL_LOCAL_FILE> = FindFileInPath(Look4This, LookWhere);
            end;

            ;--- Look relative to current directory if not found above ------
            #option PUSH AllowPack=NO
            if <$FILEINFO_RXVAR_FULL_LOCAL_FILE> = '' then
            #option POP
            do
               if LookWhere = '' then
               do
                  TryThis = Look4This;
               end;
               else
               do
                  TryThis = LookWhere || RexDirChar || Look4This;
               end;
               TryThis = ReplaceString(TryThis, RexDirChar || RexDirChar, RexDirChar);     ;;remove any double slashes (one set anyway)
               <$FILEINFO_RXVAR_FULL_LOCAL_FILE> = stream(TryThis, "c", "query exists");
            end;

            ;--- Make sure the file exists ----------------------------------
            #option PUSH AllowPack=NO
            if <$FILEINFO_RXVAR_FULL_LOCAL_FILE> = '' then
            #option POP
            do
               Error('The file "' || Look4This || '" could not be found! Looked at:', '    "' || LookWhere || '"');
            end

            ;--- Mark as input dependancy -----------------------------------
            #option PUSH AllowPack=NO
            call AddInputFileToDependancyList <$FILEINFO_RXVAR_FULL_LOCAL_FILE>;
            #option POP
#DefineRexx
#DefineRexx RexxSizeOfFile
            <$FILEINFO_RXVAR_FILE_SIZE>     = stream(<$FILEINFO_RXVAR_FULL_LOCAL_FILE>, "c", "query size");
            <$FILEINFO_RXVAR_FILE_SIZE_CMA> = AddCommasToDecimalNumber(<$FILEINFO_RXVAR_FILE_SIZE>);
#DefineRexx
#DefineRexx RexxDateOfFile
            ;--- Get file date/time -----------------------------------------
            RxFileStamp   = GetFileTimeStamp(<$FILEINFO_RXVAR_FULL_LOCAL_FILE>);

            ;--- Format the date (3 NOV 1999) -------------------------------
            RxFileDay   = substr(RxFileStamp, 7, 2) + 0; ;;Remove leading '0'
            RxFileMonth = substr(RxFileStamp, 5, 2);
           ;RxFileYear  = substr(RxFileStamp, 1, 4);    ;;4 digit
            RxFileYear  = substr(RxFileStamp, 3, 2);    ;;2 digit
            RxFileMon   = word('Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec', RxFileMonth);
            <$FILEINFO_RXVAR_FILE_DATE> = RxFileDay || '<$FILEINFO_DATE_SPACE>' || RxFileMon || '<$FILEINFO_DATE_SPACE>' || RxFileYear;
#DefineRexx
#DefineRexx RexxTimeOfFile
            ;--- Get file date/time -----------------------------------------
            RxFileStamp   = GetFileTimeStamp(<$FILEINFO_RXVAR_FULL_LOCAL_FILE>);

            ;--- Format the date (3 NOV 1999) -------------------------------
            RxFileHour    = substr(RxFileStamp, 9,  2) + 0; ;;Remove leading '0'
            RxFileMinute  = substr(RxFileStamp, 11, 2);
            if RxFileHour < 12 then
            do
               RxAmPm = '<$FILEINFO_AM>';
            end
            else
            do
               RxAmPm = '<$FILEINFO_PM>';
            end
            if RxFileHour > 12 then
            do
               RxFileHour = RxFileHour - 12;
            end;
            <$FILEINFO_RXVAR_FILE_TIME> = RxFileHour || ':' || RxFileMinute || RxAmPm;
#DefineRexx

;--- Specialized macros -----------------------------------------------------
#define   SizeOfFile                                                           \
          #evaluate+  ^^ ^<$RexxVerifyLocalInputFile {$?}>;<$RexxSizeOfFile>^ -\
          {$SFONT=^<$FILEINFO_FONT>^}                                         -\
                {$Before=""}<??<$FILEINFO_RXVAR_FILE_SIZE_CMA>> bytes{$After=""} -\
          {$EFONT=^<$FILEINFO_/FONT>^}
#define   DateOfFile                                                           \
          #evaluate+  ^^ ^<$RexxVerifyLocalInputFile {$?}>;<$RexxDateOfFile>^ -\
          {$SFONT=^<$FILEINFO_FONT>^}                                         -\
                {$Before=""}<??<$FILEINFO_RXVAR_FILE_DATE>>{$After=""}                         -\
          {$EFONT=^<$FILEINFO_/FONT>^}
#define   TimeOfFile                                                           \
          #evaluate+  ^^ ^<$RexxVerifyLocalInputFile {$?}>;<$RexxTimeOfFile>^ -\
          {$SFONT=^<$FILEINFO_FONT>^}                                         -\
                {$Before=""}<??<$FILEINFO_RXVAR_FILE_TIME>>{$After=""}                         -\
          {$EFONT=^<$FILEINFO_/FONT>^}

;--- Information in (brackets) ----------------------------------------------
#define   (SizeOfFile) <$SizeOfFile {$?} BEFORE="(" AFTER=")">
#define   (DateOfFile) <$DateOfFile {$?} BEFORE="(" AFTER=")">


;--- End of Header ----------------------------------------------------------
#endif



