;----------------------------------------------------------------------------
;     MODULE NAME:   ID3.IH
;
;         $Author:   USER "Dennis"  $
;       $Revision:   1.2  $
;           $Date:   22 Jun 2001 18:50:12  $
;        $Logfile:   C:/DBAREIS/Projects.PVCS/PpwAddOn/ID3/ID3.h.pvcs  $
;
;     DESCRIPTION:   This header allows you to easily obtain ID3 information
;                    such as song title, track etc out of MP3 files.
;
;                    It is possible that all information in and ID3 tag is
;                    blank, this is not considered to be an error, you will
;                    need to test for this...
;
;                    Note that the "FILEINFO" addon would frequently be used
;                    to find the file on the local filesystem.
;
;----------------------------------------------------------------------------


;--- Only include once! -----------------------------------------------------
#ifdef VERSION_ID3_H
       #eof 1                      ;;No point reading rest of file!
#endif
#define VERSION_ID3_H    01.173
#require 01.018

;--- Start of our our own namespace -----------------------------------------
#NextId

;--- Create a macro containing code which can read ID3 version 1 and 1.1 tags ----
#DefineRexx 'ID3'
   ;--- Get parameters ------------------------------------------------------
   @@File = '{$File}';
   @@Stem = '{$Stem=^ID3^}' || '.';    ;;User should not pass trailing '.'!

   ;--- Init values ---------------------------------------------------------
   @@Rc        = '';                        ;;"" = OK
   @@Format    = '?';                       ;;ID3 information was in this format
   @@Title     = '';
   @@Artist    = '';
   @@Album     = '';
   @@Year      = '';
   @@Comment   = '';
   @@Track     = '';
   @@Genre     = '';                        ;;The numeric code
   @@GenreDesc = '';                        ;;Test description of the code

   ;--- Read the last 128 bytes of the file ---------------------------------
   @@Lng = stream(@@File, 'c', 'query size');
   if  @@Lng <= 128 then
   do
       if  @@Lng = '' then
           @@Rc = 'File "' || @@File || '" does not exist!';
       else
           @@Rc = 'File only ' || @@Lng || ' bytes long!';
   end;
   else
   do
       ;--- Read last 128 bytes ---------------------------------------------
       @@128 = charin(@@File, @@Lng-127, 130);
       CloseRc = stream(@@File, 'c', 'close');
       if  length(@@128) <> 128 then
           @@Rc = 'Could not read 128 byte ID3 trailer';
       else
       do
           ;--- Make sure it looks like an ID3 header -----------------------
           if  left(@@128, 3) <> 'TAG' then
               @@Rc = 'File does not have ID3 header';
           else
           do
               ;--- Extract easy bits! --------------------------------------
               @@Title  = sstrip( substr(@@128,4,30) );
               @@Artist = sstrip( substr(@@128,34,30) );
               @@Album  = sstrip( substr(@@128,64,30) );
               @@Year   = sstrip( substr(@@128,94,4) );

               ;--- ID3 version 1 or 1.1? --------------------------------------------------
               @@Comment = substr(@@128,98,30);
               if substr(@@Comment, 29, 1) == '00'x & right(@@Comment, 1) \== '00'x then
               do
                   ;--- This is in ID3 V1.1 format ------------------------------------------
                   @@Format  = "1.1"
                   @@Track   = c2d(right(@@Comment, 1));
                   @@Comment = left(@@Comment, 28);
               end;
               else
               do
                   ;--- This is in ID3 V1 format --------------------------------------------
                   @@Format  = "1"
                   @@Track = "";
               end;
               @@Comment = sstrip(@@Comment);

               ;--- Handle Genre --------------------------------------------
               @@Genre = c2d(right(@@128,1));
               @@Var = 'Id3GenreDesc.' || @@Genre+1;
               if  symbol(@@Var) <> 'VAR' then
                   @@GenreDesc = 'Unknown';        ;;No description for this value
               else
                   @@GenreDesc = value(@@Var);     ;;Get value from array

               ;--- Everything's fine ---------------------------------------
               @@Rc = '';
           end;
       end;
   end;

   ;--- Update variable user indicated for results --------------------------
   call value @@Stem || 'ID3Rc',        @@Rc;
   call value @@Stem || 'ID3Format',    @@Format;
   call value @@Stem || 'ID3Title',     @@Title;
   call value @@Stem || 'ID3Artist',    @@Artist;
   call value @@Stem || 'ID3Album',     @@Album;
   call value @@Stem || 'ID3Year',      @@Year;
   call value @@Stem || 'ID3Comment',   @@Comment;
   call value @@Stem || 'ID3Track',     @@Track;
   call value @@Stem || 'ID3Genre',     @@Genre;
   call value @@Stem || 'ID3GenreDesc', @@GenreDesc;
#DefineRexx


;--- Set up all the category descriptions -----------------------------------
#DefineRexx ''
   Id3GenreDesc.0 = 0;
   call Add2 'Blues', 'Id3GenreDesc';
   call Add2 'Classic Rock';
   call Add2 'Country';
   call Add2 'Dance';
   call Add2 'Disco';
   call Add2 'Funk';
   call Add2 'Grunge';
   call Add2 'Hip-Hop';
   call Add2 'Jazz';
   call Add2 'Metal';
   call Add2 'New Age';
   call Add2 'Oldies';
   call Add2 'Other';
   call Add2 'Pop';
   call Add2 'R&B';
   call Add2 'Rap';
   call Add2 'Reggae';
   call Add2 'Rock';
   call Add2 'Techno';
   call Add2 'Industrial';
   call Add2 'Alternative';
   call Add2 'Ska';
   call Add2 'Death Metal';
   call Add2 'Pranks';
   call Add2 'Soundtrack';
   call Add2 'Euro-Techno';
   call Add2 'Ambient';
   call Add2 'Trip-Hop';
   call Add2 'Vocal';
   call Add2 'Jazz+Funk';
   call Add2 'Fusion';
   call Add2 'Trance';
   call Add2 'Classical';
   call Add2 'Instrumental';
   call Add2 'Acid';
   call Add2 'House';
   call Add2 'Game';
   call Add2 'Sound Clip';
   call Add2 'Gospel';
   call Add2 'Noise';
   call Add2 'AlternRock';
   call Add2 'Bass';
   call Add2 'Soul';
   call Add2 'Punk';
   call Add2 'Space';
   call Add2 'Meditative';
   call Add2 'Instrumental Pop';
   call Add2 'Instrumental Rock';
   call Add2 'Ethnic';
   call Add2 'Gothic';
   call Add2 'Darkwave';
   call Add2 'Techno-Industrial';
   call Add2 'Electronic';
   call Add2 'Pop-Folk';
   call Add2 'Eurodance';
   call Add2 'Dream';
   call Add2 'Southern Rock';
   call Add2 'Comedy';
   call Add2 'Cult';
   call Add2 'Gangsta';
   call Add2 'Top 40';
   call Add2 'Christian Rap';
   call Add2 'Pop/Funk';
   call Add2 'Jungle';
   call Add2 'Native American';
   call Add2 'Cabaret';
   call Add2 'New Wave';
   call Add2 'Psychadelic';
   call Add2 'Rave';
   call Add2 'Showtunes';
   call Add2 'Trailer';
   call Add2 'Lo-Fi';
   call Add2 'Tribal';
   call Add2 'Acid Punk';
   call Add2 'Acid Jazz';
   call Add2 'Polka';
   call Add2 'Retro';
   call Add2 'Musical';
   call Add2 'Rock & Roll';
   call Add2 'Hard Rock';
   call Add2 'Folk';
   call Add2 'Folk-Rock';
   call Add2 'National Folk';
   call Add2 'Swing';
   call Add2 'Fast Fusion';
   call Add2 'Bebob';
   call Add2 'Latin';
   call Add2 'Revival';
   call Add2 'Celtic';
   call Add2 'Bluegrass';
   call Add2 'Avantgarde';
   call Add2 'Gothic Rock';
   call Add2 'Progressive Rock';
   call Add2 'Psychedelic Rock';
   call Add2 'Symphonic Rock';
   call Add2 'Slow Rock';
   call Add2 'Big Band';
   call Add2 'Chorus';
   call Add2 'Easy Listening';
   call Add2 'Acoustic';
   call Add2 'Humour';
   call Add2 'Speech';
   call Add2 'Chanson';
   call Add2 'Opera';
   call Add2 'Chamber Music';
   call Add2 'Sonata';
   call Add2 'Symphony';
   call Add2 'Booty Bass';
   call Add2 'Primus';
   call Add2 'Porn Groove';
   call Add2 'Satire';
   call Add2 'Slow Jam';
   call Add2 'Club';
   call Add2 'Tango';
   call Add2 'Samba';
   call Add2 'Folklore';
   call Add2 'Ballad';
   call Add2 'Power Ballad';
   call Add2 'Rhythmic Soul';
   call Add2 'Freestyle';
   call Add2 'Duet';
   call Add2 'Punk Rock';
   call Add2 'Drum Solo';
   call Add2 'A capella';
   call Add2 'Euro-House';
   call Add2 'Dance Hall';
#DefineRexx


;--- End of our our own namespace -------------------------------------------
#NextId
