;----------------------------------------------------------------------------
;     MODULE NAME:   IMPORT_D.H
;
;         $Author:   USER "Dennis"  $
;       $Revision:   1.4  $
;           $Date:   02 May 2002 21:01:42  $
;        $Logfile:   C:/DBAREIS/Projects.PVCS/PpwAddOn/import_d/import_d.h.pvcs  $
;
;     DESCRIPTION:   This header file can be used to split a record from
;                    a delimited text file (for example tab or comma
;                    delimited) into its fields.
;
;                    This header is used in the PPWIZARD #import command and
;                    can be just as useful for you if you need to process
;                    a text file yourself.
;----------------------------------------------------------------------------


;--- Only include once! -----------------------------------------------------
#ifdef VERSION_IMPORT_D_H
       #eof 1                      ;;No point reading rest of file!
#endif
#define VERSION_IMPORT_D_H    02.122

;--- Create Name space for this header --------------------------------------
#NextId PUSH
#NextId

;--- Define how errors will be handled by default ---------------------------
#define? ImportCsvFieldsErrorsStdErrorInfo {$REASON},, 'RECORD', '~~~~~~', {$Record}, 'DETECTED AT', '~~~~~~~~~~~', {$PARTIALRECORD}
#ifndef  ImportCsvFieldsERROR
   ;--- Define default error handler (note processing is expected to 'halt')  --
   #DefineRexx 'ImportCsvFieldsERROR'
       ;--- Do WHAT on error? ("error()" available in #evaluate etc) --------
       Error(<$ImportCsvFieldsErrorsStdErrorInfo {$?}>)
   #DefineRexx
#endif

;--- What is an imbedded newline replaced with? -----------------------------
#define? IMPORT_NEWLINE '0A'x           ;;Real newline by default

;--- Define the import code -------------------------------------------------
#DefineRexx 'ImportCsvFields'
   ;--- Get passed parameters -----------------------------------------------
   @@L    = {$Record};            ;;Copy the input record
   @@Del  = {$DelChar=^','^};     ;;Delimiter
   @@MinF = {$MinFields=^0^};     ;;No minimum by default
   @@MaxF = {$MaxFields=^0^};     ;;No maximum by default

   ;--- Extract fields out of line ------------------------------------------
   @@FC   = 0;
   @@Q    = '"';
   @@Q2   = '""';
   do  while @@L <> ''
       ;--- We have another field -------------------------------------------
       @@Fc = @@Fc + 1;

       ;--- See what type of field we have ----------------------------------
       if  left(@@L,1) <> @@Q then
       do
           ;--- Just look for delimiter (field unquoted) --------------------
           @@DelPos = pos(@@Del, @@L);
           if  @@DelPos <> 0 then
           do
               ;--- Not the last field (found delimiter) --------------------
               @@F = left(@@L,  @@DelPos-1);   ;;Extract the field
               @@L = substr(@@L, @@DelPos+1);  ;;Get the rest of the line
           end;
           else
           do
               ;--- This is the last record ---------------------------------
               @@F = @@L;
               @@L = '';
           end;
       end;
       else
       do
           ;--- Have quoted field, look for end quote -----------------------
           @@LookFrom = 2;
           do  forever
               ;--- Must find a quote ---------------------------------------
               @@QPos = pos(@@Q, @@L, @@LookFrom);
               if  @@QPos = 0 then
               do
                   /*--- A quoted field can span lines ----------------------*/
                   #ifndef ImportCsvFieldsGetAnotherLine
                       /*--- No code supplied to read another line! ---------*/
                       <$ImportCsvFieldsERROR REASON=^'No ending quote on field #' || @@Fc^ Record=^{$Record}^ PARTIALRECORD="@@L">
                   #elseif
                       /*--- Read the next line into "@@LN" -----------------*/
                       <$ImportCsvFieldsGetAnotherLine RESULTVAR="@@Ln" Record=^{$Record}^ PARTIALRECORD="@@L"> ;;If can't read should generate it's own error!

                       /*--- Effectively the user has passed a "longer" record ---*/
                       {$Record} = {$Record} || @@Ln

                       /*--- Add to the end of the partial Record and retry ---*/
                       @@L = @@L || <$IMPORT_NEWLINE> || @@Ln;
                       iterate;
                   #endif
               end;

               ;--- Found a quote (if doubled up, need to ignore ------------
               if  substr(@@L, @@QPos+1, 1) = @@Q then
                   @@LookFrom = @@QPos + 2;        ;;Past 2nd quote
               else
                   leave;                          ;;Found end quote
           end;

           ;--- Save field (handle imbedded quotes) -------------------------
           @@F = ReplaceString(substr(@@L, 2, @@QPos-2), @@Q2, @@Q);
           @@L = substr(@@L, @@QPos+1);
           if  @@L <> '' then
           do
               ;--- Must have delimiter -------------------------------------
               if  left(@@L, 1) <> @@Del then
               do
                   <$ImportCsvFieldsERROR REASON=^'Expected delimiter after field #' || @@Fc^ Record=^{$Record}^ PARTIALRECORD="@@L">
               end;
               @@L = substr(@@L, 2);   ;;Remove delimiter
           end;
       end;

       ;--- Save the field --------------------------------------------------
       {$Stem}.@@Fc = @@F;

       ;--- Is there a limit to the number of fields we need? ---------------
       if  @@MaxF <> 0 then
       do
           ;--- There is a limit, have we got to it? ------------------------
           if  @@Fc >= @@MaxF then
               leave;                  ;;Exit loop as we have enough fields!
       end;
   end;

   ;--- Have minimum fields specified? --------------------------------------
   if  @@Fc < @@MinF then
   do
       ;--- See if missing fields (they are blank) --------------------------
       do  while @@Fc < @@MinF
           ;--- Pad with a blank field --------------------------------------
           @@Fc = @@Fc + 1;
           {$Stem}.@@Fc = {$MissingFieldValue=^''^};  ;;'' by default
       end;
   end;

   ;--- Update the stem count -----------------------------------------------
   {$Stem}.0 = @@Fc;
#DefineRexx


;--- Restore original Name space --------------------------------------------
#NextId POP
