/*
 *  Q&D program to convert MY OS/2 WPS URLS to Windows MY IE style
 *  WPS URL object comments are maintained...
 *
 *  This program is run under OS/2.
 */

/*--- Initialization --------------------------------------------------------*/
CmdLine = strip(arg(1));
address cmd '@echo off';
trace off;
call RxFuncAdd  'SysGetEA', 'RexxUtil', 'SysGetEA';
if CmdLine = '' then
do
   say 'ERROR: No destination directory specified';
   exit(255);
end;


/*--- Work out current directory and where to put things to -----------------*/
DestBaseDir = CmdLine     || '\';
CurrentDir  = directory() || '\';
CurrentDirL = length(CurrentDir);

/*--- Get list of all files in all subdirectories ---------------------------*/
Files.0 = 0;
call SysFileTree '.\*.*', 'Files', 'FOS';
say 'Found ' || Files.0 || ' files...'

/*--- Work through list -----------------------------------------------------*/
do Index = 1 to Files.0
   /*--- Get next file and work out where it goes in destination tree -------*/
   SrcFile    = Files.Index;
   SourcePath = filespec('drive', SrcFile) || filespec('path', SrcFile);
   Relpath    = substr(SourcePath, CurrentDirL+1);
   DstPath    = DestBaseDir || Relpath;
   DstFile    = DstPath     || filespec('name', SrcFile);
   UrlFile    = 'Y';
   DotPos     = lastpos('.', SrcFile);
   if  DotPos <> 0 then
   do
       /*--- File had an extension see if we know file is NOT a URL ---------*/
       Extn = translate(substr(SrcFile, DotPos));
       if  Extn = '.$$$' | Extn = '.DIZ' then
           UrlFile = 'N';
   end;
   if  UrlFile = 'Y' then
       DstFile = DstFile || '.url';

   /*--- Make directory -----------------------------------------------------*/
   call MakeDirectoryTree DstPath;

   /*--- Convert the URL ----------------------------------------------------*/
   call ConvertOldFile SrcFile, DstFile, UrlFile;
end;
say 'Done....';
exit(0);



/*===========================================================================*/
ConvertOldFile: procedure
/*===========================================================================*/
   /*--- Either URL file or .$$$' or '.DIZ' ---------------------------------*/
   parse arg SourceFile, DestinationFile, IsSimpleUrl;
   if  IsSimpleUrl = 'N' then
   do
       /*--- Just a simple copy will do the trick -----------------------*/
       address cmd 'copy "' || SourceFile || '" "' || DestinationFile || '" >nul'
       if  Rc <> 0 then
           say 'WARNING: copy of "' || SourceFile || '" failed RC=' || Rc;
       return(Rc);
   end;

   /*--- OK we know we have a normal WPS URL here, read URL -------------*/
   CloseRc   = stream(SourceFile, 'c', 'close');
   Url       = charin(SourceFile,, 999);
   CloseRc   = stream(SourceFile, 'c', 'close');
   Url = ChopAt('0D'x, Url);
   Url = ChopAt('0A'x, Url);
   Url = ChopAt('00'x, Url);

   /*--- Get any source comment -----------------------------------------*/
   if  SysGetEA(SourceFile, ".COMMENTS", "UrlComment") <> 0 then
       UrlComment = '';
   else
   do
       UrlComment = substr(UrlComment, 11);         /* Drop EA Type info */
       UrlComment = space(ReplaceString(UrlComment, '0D0A'x, '<P>'));
   end;

   /*--- Make sure destination does not exist ---------------------------*/
   CloseRc = stream(DestinationFile, 'c', 'close');
   address cmd 'del "' || DestinationFile || '" >nul 2>&1'

   /*--- Create a new file ----------------------------------------------*/
   call DestLine '[DEFAULT]'
   call DestLine 'BASEURL=' || Url
   call DestLine '[InternetShortcut]'
   call DestLine 'URL=' || Url;
   if  UrlComment <> '' then
   do
       call DestLine ''
       call DestLine ''
       call DestLine '[BmComments]'
       call DestLine UrlComment;
       call DestLine '[BmCommentsEnd]'
   end;
   CloseRc = stream(DestinationFile, 'c', 'close');
   return(0);



/*===========================================================================*/
DestLine:
/*===========================================================================*/
   call lineout DestinationFile, arg(1);
   return;

/*===========================================================================*/
ChopAt: procedure
/*===========================================================================*/
   parse arg ChopChar, String;
   ChopPos = pos(ChopChar, String);
   if  ChopPos = 0 then
       return(String);
   else
       return(left(String, ChopPos-1));



/*===========================================================================*/
MakeDirectoryTree: procedure
/*                                                                           */
/* ARG(1) = The name of a directory                                          */
/*                                                                           */
/* This routine will start from the specified base and build all directories */
/* required to get arg(1) to be the name of a directory which exists.        */
/*                                                                           */
/* Note currently does no check that directory exists at completion.         */
/*===========================================================================*/
   /*--- Get directory and remove any terminating slash ---------------------*/
   WholeDirectory = arg(1);
   RexDirChar     = '\';
   if  right(WholeDirectory, 1) = RexDirChar then
       WholeDirectory = left(WholeDirectory, length(WholeDirectory)-1)
   if  WholeDirectory = '' then
       return(0);

   /*--- Loop through making all 'parts' ------------------------------------*/
   MakeDirCmd        = 'md ';
   SearchFromPosn    = 1;
   do  until SlashPosn = 0
       /*--- Find the next slash --------------------------------------------*/
       SlashPosn = pos(RexDirChar, WholeDirectory, SearchFromPosn);

       /*--- Make the directory if not "\" ----------------------------------*/
       if  SlashPosn <> 1 then
       do
           /*--- Work out what we need to make ------------------------------*/
           if  SlashPosn = 0 then
               MakeDir = WholeDirectory;
           else
               MakeDir = left(WholeDirectory, SlashPosn-1);
           DirBit = filespec('name', MakeDir);

           /*--- Don't make 'C:' etc! ---------------------------------------*/
           if  right(MakeDir, 1) <> ':' & DirBit <> '.' & DirBit <> '..' then
               Address Cmd MakeDirCmd || ' "' || MakeDir || '" >nul 2>&1'
       end;

       /*--- Make sure we don't pick the same slash up next time ------------*/
       SearchFromPosn = SlashPosn + 1;
   end;
   return(0);



/*===========================================================================*/
ReplaceString:
/*===========================================================================*/
   /*--- Get the passed parameters ------------------------------------------*/
   TheString    = arg(1);
   ChangeFrom   = arg(2);
   ChangeTo     = arg(3);

   /*--- Look for the search string -----------------------------------------*/
   ChangeFromLength = length(ChangeFrom);
   ChangeToLength   = length(ChangeTo);
   FoundPosn = pos(ChangeFrom, TheString);
   do  while FoundPosn <> 0
       /*--- Perform the substitution ---------------------------------------*/
       TheString = left(TheString, FoundPosn-1) || ChangeTo || substr(TheString, FoundPosn+ChangeFromLength);

       /*--- Look for the more occurances of the search string --------------*/
       FoundPosn = pos(ChangeFrom, TheString, FoundPosn+ChangeToLength);
   end;
   return(TheString);


